from insights.parsers import iptables
from insights.parsers.iptables import IPTablesConfiguration, IPTables, IP6Tables, IPTabPermanent, IP6TabPermanent
from insights.tests import context_wrap
import doctest

IPTABLES_SAVE = """
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [769:196899]
:REJECT-LOG - [0:0]
-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
-A INPUT -m state --state RELATED,ESTABLISHED -g ACCEPT
-A INPUT -s 192.168.0.0/24 -j ACCEPT
-A INPUT -s 192.168.2.0/24
-A INPUT -p icmp -j ACCEPT
-A INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
-A INPUT -j REJECT --reject-with icmp-host-prohibited
-A REJECT-LOG -p tcp -j REJECT --reject-with tcp-reset
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*mangle
:PREROUTING ACCEPT [451:22060]
:INPUT ACCEPT [451:22060]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [594:47151]
:POSTROUTING ACCEPT [594:47151]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*nat
:PREROUTING ACCEPT [0:0]
:POSTROUTING ACCEPT [3:450]
:OUTPUT ACCEPT [3:450]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
"""

PARSED_TCP_REJECT_RULE = {
    "table": "filter",
    "chain": "REJECT-LOG",
    "rule": "-p tcp -j REJECT --reject-with tcp-reset",
    "target_action": "jump",
    "constraints": "-p tcp",
    "target": "REJECT",
    "target_options": "--reject-with tcp-reset"
}


def check_iptables_rules_parsing(iptables_obj):
    ipt = iptables_obj
    assert len(ipt.rules) == 8
    assert len(ipt.get_chain("INPUT")) == 7
    assert len(ipt.table_chains("mangle")) == 5
    assert ipt.rules[-1] == PARSED_TCP_REJECT_RULE
    assert ipt.get_table("nat")[1] == {
        "policy": "ACCEPT",
        "table": "nat",
        "name": "POSTROUTING",
        "packet_counter": 3,
        "byte_counter": 450
    }
    assert "tcp-reset" in ipt
    assert "--sport" not in ipt
    assert ipt.get_rule("tcp-reset") == [PARSED_TCP_REJECT_RULE]


def test_iptables_configuration():
    ipt = IPTablesConfiguration(context_wrap(IPTABLES_SAVE))
    check_iptables_rules_parsing(ipt)


def test_iptables_save():
    ipt = IPTables(context_wrap(IPTABLES_SAVE))
    check_iptables_rules_parsing(ipt)


def test_iptables_permanent():
    ipt = IPTabPermanent(context_wrap(IPTABLES_SAVE))
    check_iptables_rules_parsing(ipt)


IP6TABLES_SAVE = """
# Generated by ip6tables-save v1.4.21 on Tue Jan 31 05:25:29 2017
*nat
:PREROUTING ACCEPT [0:0]
:INPUT ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:POSTROUTING ACCEPT [0:0]
COMMIT
# Completed on Tue Jan 31 05:25:29 2017
# Generated by ip6tables-save v1.4.21 on Tue Jan 31 05:25:29 2017
*mangle
:PREROUTING ACCEPT [0:0]
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:POSTROUTING ACCEPT [0:0]
COMMIT
# Completed on Tue Jan 31 05:25:29 2017
# Generated by ip6tables-save v1.4.21 on Tue Jan 31 05:25:29 2017
*security
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [8:512]
:FORWARD_direct - [0:0]
:INPUT_direct - [0:0]
:OUTPUT_direct - [0:0]
-A INPUT -j INPUT_direct
-A FORWARD -j FORWARD_direct
-A OUTPUT -j OUTPUT_direct
COMMIT
# Completed on Tue Jan 31 05:25:29 2017
# Generated by ip6tables-save v1.4.21 on Tue Jan 31 05:25:29 2017
*raw
:PREROUTING ACCEPT [0:0]
:OUTPUT ACCEPT [8:512]
:OUTPUT_direct - [0:0]
:PREROUTING_direct - [0:0]
-A PREROUTING -p ipv6-icmp -m icmp6 --icmpv6-type 134 -j ACCEPT
-A PREROUTING -m rpfilter --invert -j DROP
-A PREROUTING -j PREROUTING_direct
-A OUTPUT -j OUTPUT_direct
COMMIT
# Completed on Tue Jan 31 05:25:29 2017
# Generated by ip6tables-save v1.4.21 on Tue Jan 31 05:25:29 2017
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:REJECT-LOG - [0:0]
-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
-A INPUT -s fe80::/64 -j ACCEPT
-A INPUT -s fe80::/64
-A INPUT -p ipv6-icmp -j ACCEPT
-A INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
-A INPUT -j REJECT --reject-with icmp6-adm-prohibited
-A REJECT-LOG -p tcp -j REJECT --reject-with tcp-reset
COMMIT
# Completed on Tue Jan 31 05:25:29 2017
"""


PARSED_TCP_REJECT_RULE_6 = {
    'table': 'filter',
    'chain': 'REJECT-LOG',
    'rule': '-p tcp -j REJECT --reject-with tcp-reset',
    'target_action': 'jump',
    'constraints': '-p tcp',
    'target': 'REJECT',
    'target_options': '--reject-with tcp-reset',
}


def check_ip6tables_rules_parsing(ip6tables_obj):
    ipt = ip6tables_obj
    assert len(ipt.rules) == 14
    assert len(ipt.get_chain("INPUT")) == 6
    assert len(ipt.table_chains("mangle")) == 5
    assert ipt.rules[-1] == PARSED_TCP_REJECT_RULE_6
    assert ipt.get_table("nat")[-1] == {
        'policy': 'ACCEPT',
        'table': 'nat',
        'byte_counter': 0,
        'name': 'POSTROUTING',
        'packet_counter': 0,
    }
    assert "tcp-reset" in ipt
    assert "--sport" not in ipt
    assert ipt.get_rule("tcp-reset") == [PARSED_TCP_REJECT_RULE]


def test_ip6tables_save():
    ipt = IP6Tables(context_wrap(IP6TABLES_SAVE))
    check_ip6tables_rules_parsing(ipt)


def test_ip6tables_permanent():
    ipt = IP6TabPermanent(context_wrap(IP6TABLES_SAVE))
    check_ip6tables_rules_parsing(ipt)


IPTABLES_SAVE_DOC = """
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [769:196899]
:REJECT-LOG - [0:0]
-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
-A INPUT -s 192.168.0.0/24 -j ACCEPT
-A INPUT -p icmp -j ACCEPT
-A INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
-A INPUT -j REJECT --reject-with icmp-host-prohibited
-A REJECT-LOG -p tcp -j REJECT --reject-with tcp-reset
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*mangle
:PREROUTING ACCEPT [451:22060]
:INPUT ACCEPT [451:22060]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [594:47151]
:POSTROUTING ACCEPT [594:47151]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*nat
:PREROUTING ACCEPT [0:0]
:POSTROUTING ACCEPT [3:450]
:OUTPUT ACCEPT [3:450]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
"""


def test_ip6tables_doc():
    env = {
            'IPTables': IPTables,
            'ipt': IPTables(context_wrap(IPTABLES_SAVE_DOC)),
          }
    failed, total = doctest.testmod(iptables, globs=env)
    assert failed == 0


IP6TABLES_SAVE_SPECIAL = """
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [769:196899]
:REJECT-LOG - [0:0]
:Drop - [0:0]
-A INPUT -j REJECT --reject-with icmp6-adm-prohibited
-A OUTPUT -d 192.168.0.23/32 -m comment --comment "Permit IP to device net-j" -j ACCEPT
-A DROP
-A REJECT-LOG -p tcp -j REJECT --reject-with tcp-reset
-A INPUT -p tcp -m tcp --dport 11 -m conntrack --ctstate NEW -j {s_1}
# Just a test to cover 100->101
-A
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*mangle
:PREROUTING ACCEPT [451:22060]
:INPUT ACCEPT [451:22060]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [594:47151]
:POSTROUTING ACCEPT [594:47151]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
# Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
*nat
:PREROUTING ACCEPT [0:0]
:POSTROUTING ACCEPT [3:450]
:OUTPUT ACCEPT [3:450]
COMMIT
# Completed on Tue Aug 16 10:18:43 2016
"""

PARSED_TCP_REJECT_RULEs = [
    {
        'table': 'filter',
        'chain': 'INPUT',
        'rule': '-j REJECT --reject-with icmp6-adm-prohibited',
        'target_action': 'jump',
        'constraints': '',
        'target': 'REJECT',
        'target_options': '--reject-with icmp6-adm-prohibited',
    },
    {
        'table': 'filter',
        'chain': 'OUTPUT',
        'rule': '-d 192.168.0.23/32 -m comment --comment "Permit IP to device net-j" -j ACCEPT',
        'target': 'ACCEPT',
        'target_options': None,
        'target_action': 'jump',
        'constraints': '-d 192.168.0.23/32 -m comment --comment "Permit IP to device net-j"'
    },
    {
        'table': 'filter',
        'rule': '',
        'chain': 'DROP'
    },
    {
        'table': 'filter',
        'chain': 'REJECT-LOG',
        'rule': '-p tcp -j REJECT --reject-with tcp-reset',
        'target': 'REJECT',
        'target_options': '--reject-with tcp-reset',
        'target_action': 'jump',
        'constraints': '-p tcp'
    },
    {
        'table': 'filter',
        'chain': 'INPUT',
        'rule': '-p tcp -m tcp --dport 11 -m conntrack --ctstate NEW -j',
        'target': '',
        'target_options': None,
        'target_action': 'jump',
        'constraints': '-p tcp -m tcp --dport 11 -m conntrack --ctstate NEW'
    }
]


def test_ip6tables_save_special():
    ipt = IP6Tables(context_wrap(IP6TABLES_SAVE_SPECIAL.format(s_1='')))
    assert len(ipt.rules) == 5
    assert len(ipt.get_chain("INPUT")) == 2
    assert len(ipt.get_chain("OUTPUT")) == 1
    for rst, exp in zip(ipt.rules, PARSED_TCP_REJECT_RULEs):
        assert rst == exp
    assert ipt.get_chain("DROP") == [
                {'table': 'filter', 'rule': '', 'chain': 'DROP'}]



"""
Heat logs
=========

Module for parsing the log files for Heat.  Parsers included are:

HeatApiLog - file ``/var/log/heat/heat-api.log, /var/log/containers/heat/heat_api.log``
---------------------------------------------------------------------------------------

HeatEngineLog - file ``/var/log/heat/heat-engine.log``
------------------------------------------------------

"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.heat_api_log)
class HeatApiLog(LogFileOutput):
    """Class for parsing ``/var/log/heat/heat-api.log, /var/log/containers/heat/heat_api.log`` file.

    Typical content of ``heat-api.log`` file is::

        2016-11-09 14:39:29.223 3844 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
        2016-11-09 14:39:30.612 3844 INFO heat.api [-] Starting Heat REST API on 172.16.2.12:8004
        2016-11-09 14:39:30.612 3844 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
        2016-11-09 14:39:30.615 3844 INFO heat.common.wsgi [-] Starting 0 workers
        2016-11-09 14:39:30.625 3844 INFO heat.common.wsgi [-] Started child 4136
        2016-11-09 14:39:30.641 3844 INFO heat.common.wsgi [-] Started child 4137
        2016-11-09 14:39:30.723 4137 INFO eventlet.wsgi.server [-] (4137) wsgi starting up on http://172.16.2.12:8004
        2016-11-09 14:39:30.728 3844 INFO heat.common.wsgi [-] Started child 4139
        2016-11-09 14:39:30.750 4140 INFO eventlet.wsgi.server [-] (4140) wsgi starting up on http://172.16.2.12:8004
        2016-11-09 14:39:30.732 3844 INFO heat.common.wsgi [-] Started child 4140
        2016-11-09 14:39:30.764 4139 INFO eventlet.wsgi.server [-] (4139) wsgi starting up on http://172.16.2.12:8004
        2016-11-09 14:39:30.782 4136 INFO eventlet.wsgi.server [-] (4136) wsgi starting up on http://172.16.2.12:8004

    .. note::
        Please refer to its super-class :py:class:`insights.core.LogFileOutput`
    """
    pass


@parser(Specs.heat_engine_log)
class HeatEngineLog(LogFileOutput):
    """Class for parsing ``/var/log/heat/heat-engine.log`` file.

    Typical content of ``heat-engine.log`` file is::

        2016-11-09 14:32:43.062 4392 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
        2016-11-09 14:32:43.371 4392 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
        2016-11-09 14:32:43.374 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.monasca: "No module named monascaclient". Not using monasca.
        2016-11-09 14:32:43.402 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.senlin: "No module named senlinclient". Not using senlin.
        2016-11-09 14:32:43.761 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.senlin: "No module named senlinclient". Not using senlin.cluster.
        2016-11-09 14:32:43.763 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.senlin: "No module named senlinclient". Not using senlin.profile.
        2016-11-09 14:32:43.763 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.monasca: "No module named monascaclient". Not using monasca.notification.
        2016-11-09 14:32:43.764 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.senlin: "No module named senlinclient". Not using senlin.profile_type.
        2016-11-09 14:32:43.765 4392 WARNING heat.common.pluginutils [-] Encountered exception while loading heat.engine.clients.os.senlin: "No module named senlinclient". Not using senlin.policy_type.
        2016-11-09 14:32:44.153 4392 WARNING heat.engine.environment [-] OS::Aodh::CombinationAlarm is DEPRECATED. The combination alarm is deprecated and disabled by default in Aodh.
        2016-11-09 14:32:44.154 4392 WARNING heat.engine.environment [-] OS::Heat::HARestarter is DEPRECATED. The HARestarter resource type is deprecated and will be removed in a future release of Heat, once it has support for auto-healing any type of resource. Note that HARestarter does *not* actually restart servers - it deletes and then recreates them. It also does the same to all dependent resources, and may therefore exhibit unexpected and undesirable behaviour. Instead, use the mark-unhealthy API to mark a resource as needing replacement, and then a stack update to perform the replacement while respecting  the dependencies and not deleting them unnecessarily.
        2016-11-09 14:32:44.154 4392 WARNING heat.engine.environment [-] OS::Heat::SoftwareDeployments is HIDDEN. Please use OS::Heat::SoftwareDeploymentGroup instead.
        2016-11-09 14:32:44.155 4392 WARNING heat.engine.environment [-] OS::Heat::StructuredDeployments is HIDDEN. Please use OS::Heat::StructuredDeploymentGroup instead.
        2016-11-09 14:32:44.156 4392 WARNING heat.engine.environment [-] OS::Neutron::ExtraRoute is UNSUPPORTED. Use this resource at your own risk.

    .. note::
        Please refer to its super-class :py:class:`insights.core.LogFileOutput`
    """
    pass



"""
DMesgLineList - command ``dmesg``
=================================

DMesgLineList is a simple parser that is based on the ``LogFileOutput``
parser class.

It provides one additional helper method not included in ``LogFileOutput``:

* ``has_startswith`` - does the log contain any line that starts with the
  given string?

Sample input::

[    0.000000] Linux version 3.10.0-327.36.3.el7.x86_64 (mockbuild@x86-037.build.eng.bos.redhat.com) (gcc version 4.8.5 20150623 (Red Hat 4.8.5-4) (GCC) ) #1 SMP Thu Oct 20 04:56:07 EDT 2016
[    0.000000] Command line: BOOT_IMAGE=/vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/RHEL7CSB/Root ro rd.lvm.lv=RHEL7CSB/Root rd.luks.uuid=luks-96c66446-77fd-4431-9508-f6912bd84194 crashkernel=auto vconsole.keymap=us rd.lvm.lv=RHEL7CSB/Swap vconsole.font=latarcyrheb-sun16 rhgb quiet LANG=en_GB.utf8
[    0.000000] PID hash table entries: 4096 (order: 3, 32768 bytes)
[    0.000000] x86/fpu: xstate_offset[2]: 0240, xstate_sizes[2]: 0100
[    0.000000] xsave: enabled xstate_bv 0x7, cntxt size 0x340
[    0.000000] AGP: Checking aperture...
[    0.000000] AGP: No AGP bridge found
[    0.000000] Memory: 15918544k/17274880k available (6444k kernel code, 820588k absent, 535748k reserved, 4265k data, 1632k init)
[    0.000000] SLUB: HWalign=64, Order=0-3, MinObjects=0, CPUs=8, Nodes=1
[    0.000000] Hierarchical RCU implementation.

Examples:

    >>> dmesg = shared[DmesgLineList]
    >>> 'BOOT_IMAGE' in dmesg
    True
    >>> dmesg.get('AGP')
    ['[    0.000000] AGP: Checking aperture...', '[    0.000000] AGP: No AGP bridge found']
"""

from .. import LogFileOutput, parser, CommandParser
from ..specs import Specs
import re


@parser(Specs.dmesg)
class DmesgLineList(CommandParser, LogFileOutput):
    """
    Class for reading output of ``dmesg`` using the LogFileOutput parser class.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`

    """
    _line_re = re.compile(r'^(?:\[\s*(?P<timestamp>\d+\.\d+)\]\s+)?(?P<message>.*)$')

    def has_startswith(self, prefix):
        """
        Parameters:
            prefix (str): The prefix of the line to look for. Ignores any
                timestamp before the message body.

        Returns:
            (bool): Does any line start with the given prefix?
        """
        return any(
            self._line_re.search(line).group('message').startswith(prefix)
            for line in self.lines
        )

    def logs_startwith(self, prefix):
        """
        Parameters:
            prefix (str): The prefix of the line to look for. Ignores any timestamp.

        Returns:
            (list): All logs start with the given prefix without timestamp.
        """
        msgs = list()
        for line in self.lines:
            match = self._line_re.search(line)
            if match:
                msg = match.group('message')
                if msg.startswith(prefix):
                    msgs.append(msg)
        return msgs

    def get_after(self, timestamp, s=None):
        """
        Find all the (available) logs that are after the given time stamp.

        If `s` is not supplied, then all lines are used.  Otherwise, only the
        lines contain the `s` are used.  `s` can be either a single string or a
        strings list. For list, all keywords in the list must be found in the
        line.

        .. note::
            The time stamp is the floating point number of seconds
            after the boot time, and is not related to an actual datetime.  If
            a time stamp is not found on the line between square brackets, then
            it is treated as a continuation of the previous line and is only
            included if the previous line's timestamp is greater than the
            timestamp given.  Because continuation lines are only included if a
            previous line has matched, this means that searching in logs that do
            not have a time stamp produces no lines.

        Parameters:
            timestamp(float): log lines after this time are returned.
            s(str or list): one or more strings to search for.
                If not supplied, all available lines are searched.

        Yields:
            Log lines with time stamps after the given time.

        Raises:
            TypeError: The ``timestamp`` should be in `float` type, otherwise a
                `TypeError` will be raised.
        """
        if not isinstance(timestamp, float):
            raise TypeError(
                        "get_after needs a float type timestamp, but get '{c}'".format(
                            c=timestamp)
                    )

        including_lines = False
        search_by_expression = self._valid_search(s)
        for line in self.lines:
            # If `s` is not None, keywords must be found in the line
            if s and not search_by_expression(line):
                continue
            # Otherwise, search all lines
            match = self._line_re.search(line)
            if match and match.group('timestamp'):
                # Get logtimestamp and compare to given timestamp
                logstamp = float(match.group('timestamp'))
                if logstamp >= timestamp:
                    including_lines = True
                    yield self._parse_line(line)
                else:
                    including_lines = False
            else:
                # If we're including lines, add this continuation line
                if including_lines:
                    yield self._parse_line(line)



import doctest
import pytest
from insights.parsers import firewall_config
from insights.parsers.firewall_config import FirewallDConf
from insights.tests import context_wrap
from insights.parsers import SkipException

FIREWALLD_CONFIG = """
# firewalld config file

# default zone
# The default zone used if an empty zone string is used.
# Default: public
DefaultZone=public

# Minimal mark
# Marks up to this minimum are free for use for example in the direct
# interface. If more free marks are needed, increase the minimum
# Default: 100
MinimalMark=100

# Clean up on exit
# If set to no or false the firewall configuration will not get cleaned up
# on exit or stop of firewalld
# Default: yes
CleanupOnExit=yes


""".strip()

FIREWALLD_CONFIG_2 = """
""".strip()


def test_docs():
    env = {
        'firewalld': FirewallDConf(context_wrap(FIREWALLD_CONFIG))
    }
    failed, total = doctest.testmod(firewall_config, globs=env)
    assert failed == 0


def test_empty_content():
    with pytest.raises(SkipException):
        FirewallDConf(context_wrap(FIREWALLD_CONFIG_2))



"""
NginxConf - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files
==============================================================================

Parse the keyword-and-value of an Nginx configuration file.

Generally, each line is split on the first space into key and value, leading
and trailing space being ignored.

Example nginx.conf file::

    user       root
    worker_processes  5;
    error_log  logs/error.log;
    pid        logs/nginx.pid;
    worker_rlimit_nofile 8192;

    events {
      worker_connections  4096;
    }

    mail {
      server_name mail.example.com;
      auth_http  localhost:9000/cgi-bin/auth;
      server {
        listen   143;
        protocol imap;
      }
    }

    http {
      include  /etc/nginx/conf.d/*.conf
      index    index.html index.htm index.php;

      default_type application/octet-stream;
      log_format   main '$remote_addr - $remote_user [$time_local]  $status '
                        '"$request" $body_bytes_sent "$http_referer" '
                        '"$http_user_agent" "$http_x_forwarded_for"';
      access_log   logs/access.log  main;
      sendfile     on;
      tcp_nopush   on;
      server_names_hash_bucket_size 128;

      server { # php/fastcgi
        listen       80;
        server_name  domain1.com www.domain1.com;
        access_log   logs/domain1.access.log  main;
        root         html;

        location ~ \.php$ {
          fastcgi_pass   127.0.0.1:1025;
        }
      }

      server { # simple reverse-proxy
        listen       80;
        server_name  domain2.com www.domain2.com;
        access_log   logs/domain2.access.log  main;

        location ~ ^/(images|javascript|js|css|flash|media|static)/  {
          root    /var/www/virtual/big.server.com/htdocs;
          expires 30d;
        }

        location / {
          proxy_pass   http://127.0.0.1:8080;
        }
      }

      map $http_upgrade $connection_upgrade {
        default upgrade;
        '' close;
      }

      upstream websocket {
        server 10.66.208.205:8010;
      }

      upstream big_server_com {
        server 127.0.0.3:8000 weight=5;
        server 127.0.0.3:8001 weight=5;
        server 192.168.0.1:8000;
        server 192.168.0.1:8001;
      }

      server { # simple load balancing
        listen          80;
        server_name     big.server.com;
        access_log      logs/big.server.access.log main;

        location / {
          proxy_pass      http://big_server_com;
        }
      }
    }

Examples:

    >>> nginxconf = NginxConf(context_wrap(NGINXCONF))
    >>> nginxconf['user']
    'root'
    >>> nginxconf['events']['worker_connections'] # Values are all kept as strings.
    '4096'
    >>> nginxconf['mail']['server'][0]['listen']
    '143'
    >>> nginxconf['http']['access_log']
    'logs/access.log  main'
    >>> nginxconf['http']['server'][0]['location'][0]['fastcgi_pass']
    '127.0.0.1:1025'
"""

from .. import parser, LegacyItemAccess, Parser, get_active_lines
from ..specs import Specs
from insights.contrib.nginxparser import create_parser, UnspacedList
from insights.util import deprecated


@parser(Specs.nginx_conf)
class NginxConf(Parser, LegacyItemAccess):
    """
    .. warning::
        This parser is deprecated, please import
        :py:class:`insights.combiners.nginx_conf.NginxConfTree` instead.

    Class for ``nginx.conf`` and ``conf.d`` configuration files.

    Gerenally nginx.conf is writed as key-value format. It has a mail section and several sections,
    http, mail, events, etc. They are unique, and subsection server and location in http section could
    be duplicate, so the value of these subsections may be list.
    """
    def __init__(self, *args, **kwargs):
        deprecated(NginxConf, "Import NginxConfTree from insights.combiners.nginx_conf instead")
        super(NginxConf, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        list_result = UnspacedList(create_parser().parseString("\n".join(get_active_lines(content))).asList())
        self.data = self._convert_nginx_list_to_dict(list_result)

    def _convert_nginx_list_to_dict(self, li):
        """
        After parsed by create_parser(), the result is a list, it is better to convert to dict for convenience.
        """

        def _list_depth_to_dict(self, li):
            """
            Function to convert list whose depth is tree to dict. Generally, the section name would be a dict key, and content
            embraced by brace would be value. In some sections, the first item is like ['location', '/'], in this case, the
            convert rule is that 'location' would be the key, and add {"name":'/'} key-value to the dict value.
            """
            dict_ret = {}
            if self._depth(li) == 1:
                return self._handle_key_value(dict_ret, li[0], li[1])
            else:
                for sub_item in li[1]:
                    tmp_dict = _list_depth_to_dict(self, sub_item)
                    tmp_key = list(tmp_dict.keys())[0]
                    tmp_val = tmp_dict[tmp_key]
                    tmp_val = tmp_val[0] if isinstance(tmp_val, list) else tmp_val
                    dict_ret.update(self._handle_key_value(dict_ret, tmp_key, tmp_val))
                if len(li[0]) > 1:
                    dict_ret["name"] = ' '.join(li[0][1:])
                return {li[0][0]: dict_ret}

        dict_all = {}
        for item in li:
            dict_all.update(_list_depth_to_dict(self, item))
        return dict_all

    def _handle_key_value(self, t_dict, key, value):
        """
        Function to handle dict key has multi value, and return the values as list.
        """
        # As it is possible that key "server", "location", "include" and "upstream" have multi value, set the value of dict as list.
        if key in ("server", "location", "include", "upstream"):
            if key in t_dict:
                val = t_dict[key]
                val.append(value)
                return {key: val}
            return {key: [value]}
        else:
            return {key: value}

    def _depth(self, l):
        """
        Function to count the depth of a list
        """
        if isinstance(l, list) and len(l) > 0:
            return 1 + max(self._depth(item) for item in l)
        else:
            return 0



"""
Cinder configuration - file ``/etc/cinder/cinder.conf``
=======================================================

The Cinder configuration file is a standard '.ini' file and this parser uses
the ``IniConfigFile`` class to read it.

Sample configuration::

    [DEFAULT]
    rpc_backend=cinder.openstack.common.rpc.impl_kombu
    control_exchange=openstack

    osapi_volume_listen=10.22.100.58
    osapi_volume_workers=32

    api_paste_config=/etc/cinder/api-paste.ini
    glance_api_servers=http://10.22.120.50:9292
    glance_api_version=2
    glance_num_retries=0
    glance_api_insecure=False
    glance_api_ssl_compression=False

    enable_v1_api=True
    enable_v2_api=True
    storage_availability_zone=nova
    default_availability_zone=nova
    enabled_backends=tripleo_ceph
    nova_catalog_info=compute:Compute Service:publicURL
    nova_catalog_admin_info=compute:Compute Service:adminURL

    [lvm]
    iscsi_helper=lioadm
    volume_group=cinder-volumes
    iscsi_ip_address=192.168.88.10
    volume_driver=cinder.volume.drivers.lvm.LVMVolumeDriver
    volumes_dir=/var/lib/cinder/volumes
    iscsi_protocol=iscsi
    volume_backend_name=lvm

Examples:

    >>> conf = shared[CinderConf]
    >>> conf.sections()
    ['DEFAULT', 'lvm']
    >>> 'lvm' in conf
    True
    >>> conf.has_option('DEFAULT', 'enabled_backends')
    True
    >>> conf.get("DEFAULT", "enabled_backends")
    "tripleo_ceph"
    >>> conf.get("DEFAULT", "glance_api_ssl_compression")
    "False"
    >>> conf.getboolean("DEFAULT", "glance_api_ssl_compression")
    False
    >>> conf.getint("DEFAULT", "glance_aip_version")
    2

"""

from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.cinder_conf)
class CinderConf(IniConfigFile):
    """
    Cinder configuration parser class, based on the ``IniConfigFile`` class.
    """
    pass



from __future__ import absolute_import
from insights.parsers.lvm import Pvs, PvsHeadings
from insights.tests import context_wrap
from .test_lvm import compare_partial_dicts

FD_LEAK_HEADER = "File descriptor 5 (/dev/null) leaked on invocation. Parent PID 99999: timeout\n"

PVS_INFO = """
    WARNING: Locking disabled. Be careful! This could corrupt your metadata.
    LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='500.00m'|LVM2_PV_NAME='/dev/sda1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='1'
    LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='JvSULk-ileq-JbuS-GGgg-jkif-thuW-zvFBEl'|LVM2_DEV_SIZE='476.45g'|LVM2_PV_NAME='/dev/sda2'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='1020.00k'|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='476.45g'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='476.45g'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='121971'|LVM2_PV_PE_ALLOC_COUNT='121970'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '

""".strip()

PVS_SDA2_INFO = {
    'Fmt': 'lvm2',
    'PV': '/dev/sda2',
    'PSize': '476.45g',
    'PFree': '4.00m',
    'Attr': 'a--',
    'VG': None,
    '#PMda': '1',
    '#PMdaUse': '1',
    '1st_PE': '1.00m',
    'Alloc': '121970',
    'Allocatable': 'allocatable',
    'BA_size': '0',
    'BA_start': '0',
    'DevSize': '476.45g',
    'Exported': '',
    'Missing': '',
    'PE': '121971',
    'PMdaFree': '0',
    'PMdaSize': '1020.00k',
    'PV_Tags': '',
    'PV_UUID': 'JvSULk-ileq-JbuS-GGgg-jkif-thuW-zvFBEl',
    'Used': '476.45g'
}

PVS_INFO_LONG = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data1/lv_brick1'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='28'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data1/lv_hdfs1'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='41'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data2/lv_brick2'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='30'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data2/lv_hdfs2'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='42'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data3/lv_brick3'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='32'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data3/lv_hdfs3'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='43'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data4/lv_brick4'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='34'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data4/lv_hdfs4'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='44'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data5/lv_brick5'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='36'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data5/lv_hdfs5'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='45'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data6/lv_brick6'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='38'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data6/lv_hdfs6'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='46'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data7/lv_brick7'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='40'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data7/lv_hdfs7'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='47'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='100.00g'|LVM2_PV_NAME='/dev/loop0'|LVM2_PV_MAJOR='7'|LVM2_PV_MINOR='0'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='2.00g'|LVM2_PV_NAME='/dev/loop1'|LVM2_PV_MAJOR='7'|LVM2_http://m.arkansasonline.com/obituaries/2017/apr/10/charles-dietz-2017-04-10/PV_MINOR='1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='100.00g'|LVM2_PV_NAME='/dev/mapper/docker-253:3-100685290-pool'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='5'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='976.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/home'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='2'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='2.57t'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/opt'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='4'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='50.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/root'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='0'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='16.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/swap'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='50.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/var'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='3'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='1.00g'|LVM2_PV_NAME='/dev/sda2'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='2'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='3e2mRe-1y3a-iUSj-F3tH-yG1M-t3qD-8uNDf5'|LVM2_DEV_SIZE='3.63t'|LVM2_PV_NAME='/dev/sda3'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='3'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='1020.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='3.63t'|LVM2_PV_FREE='0 '|LVM2_PV_USED='3.63t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='952831'|LVM2_PV_PE_ALLOC_COUNT='952831'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='rhel_ceehadoop1'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdb1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='17'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data1'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='sxTsQE-raAM-8s3b-FSy0-LhL2-Naue-f2ebcA'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdc1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='33'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data2'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='yxj2qs-qOKA-57XF-228r-qWQj-3zez-i0Vaiw'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdd1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='49'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data3'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='94oczR-Utni-dOzj-3CMQ-efOJ-TVa8-yak25P'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sde1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='65'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data4'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='r3UW8H-pLz5-rpsn-GK2u-1kJE-GKsy-uOeoIz'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdf1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='81'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data5'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='oZtJSv-HM9w-4She-1TU0-Z31G-dgKX-liJNID'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdg1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='97'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data6'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='Ys0p6S-WJOW-TAZT-czmX-iOYq-0Tgg-v4cl0H'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdh1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='113'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data7'
""".strip()   # noqa: W291

PVS_INFO_DUP = """
WARNING: Locking disabled. Be careful! This could corrupt your metadata.
WARNING: Device for PV mn5KxB-YKlY-u4hK-zuZJ-Ia6r-3dTg-8IDjsM not found or rejected by a filter.
WARNING: Device for PV V4xZ9b-FXOz-CrRA-Eu2e-8iOS-9EDF-YZYftK not found or rejected by a filter.
WARNING: Device for PV mn5KxB-YKlY-u4hK-zuZJ-Ia6r-3dTg-8IDjsM not found or rejected by a filter.
WARNING: Device for PV V4xZ9b-FXOz-CrRA-Eu2e-8iOS-9EDF-YZYftK not found or rejected by a filter.
LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='mn5KxB-YKlY-u4hK-zuZJ-Ia6r-3dTg-8IDjsM'|LVM2_DEV_SIZE='0 '|LVM2_PV_NAME='unknown device'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='9.51g'|LVM2_PV_FREE='0 '|LVM2_PV_USED='9.51g'|LVM2_PV_ATTR='a-m'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING='missing'|LVM2_PV_PE_COUNT='2434'|LVM2_PV_PE_ALLOC_COUNT='2434'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_VG_NAME='rhel'
LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='V4xZ9b-FXOz-CrRA-Eu2e-8iOS-9EDF-YZYftK'|LVM2_DEV_SIZE='0 '|LVM2_PV_NAME='unknown device'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='196.00m'|LVM2_PV_FREE='96.00m'|LVM2_PV_USED='100.00m'|LVM2_PV_ATTR='a-m'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING='missing'|LVM2_PV_PE_COUNT='49'|LVM2_PV_PE_ALLOC_COUNT='25'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_VG_NAME='vgtest'
""".strip()    # noqa: W291

PVS_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
    Scanning all devices to update lvmetad.
    No PV label found on /dev/loop0.
    No PV label found on /dev/loop1.
    No PV label found on /dev/sda1.
    No PV label found on /dev/fedora/root.
    No PV label found on /dev/sda2.
    No PV label found on /dev/fedora/swap.
    No PV label found on /dev/fedora/home.
    No PV label found on /dev/mapper/docker-253:1-2361272-pool.
    Wiping internal VG cache
    Wiping cache of LVM-capable devices
  PV                                                    VG     Fmt  Attr PSize   PFree DevSize PV UUID                                PMdaFree  PMdaSize  #PMda #PMdaUse PE
  /dev/fedora/home                                                  ---       0     0  418.75g                                               0         0      0        0      0
  /dev/fedora/root                                                  ---       0     0   50.00g                                               0         0      0        0      0
  /dev/fedora/swap                                                  ---       0     0    7.69g                                               0         0      0        0      0
  /dev/loop0                                                        ---       0     0  100.00g                                               0         0      0        0      0
  /dev/loop1                                                        ---       0     0    2.00g                                               0         0      0        0      0
  /dev/mapper/docker-253:1-2361272-pool                             ---       0     0  100.00g                                               0         0      0        0      0
  /dev/mapper/luks-7430952e-7101-4716-9b46-786ce4684f8d fedora lvm2 a--  476.45g 4.00m 476.45g FPLCRf-d918-LVL7-6e3d-n3ED-aiZv-EesuzY        0   1020.00k     1        1 121970
  /dev/sda1                                                         ---       0     0  500.00m                                               0         0      0        0      0
  /dev/sda2                                                         ---       0     0  476.45g                                               0         0      0        0      0
    Reloading config files
    Wiping internal VG cache
""".strip()

PVS_HEADINGS_6 = {
    'PV': '/dev/mapper/luks-7430952e-7101-4716-9b46-786ce4684f8d',
    'VG': 'fedora',
    'Fmt': 'lvm2',
    'Attr': 'a--',
    'PSize': '476.45g',
    'PFree': '4.00m',
    'DevSize': '476.45g',
    'PV_UUID': 'FPLCRf-d918-LVL7-6e3d-n3ED-aiZv-EesuzY',
    'PMdaFree': '0',
    'PMdaSize': '1020.00k',
    '#PMda': '1',
    '#PMdaUse': '1',
    'PE': '121970'
}


def test_pvs():
    def check(pvs_records):
        assert len(list(pvs_records)) == 2
        for k, v in PVS_SDA2_INFO.items():
            assert pvs_records.data["content"][1][k] == v
        assert pvs_records["/dev/sda1"]["Attr"] == "---"
        assert pvs_records.data["content"][0]['LVM2_PV_MINOR'] == '1'

    check(Pvs(context_wrap(PVS_INFO)))
    check(Pvs(context_wrap(FD_LEAK_HEADER + PVS_INFO)))

    pvs_records = Pvs(context_wrap(PVS_INFO_LONG))
    assert len(list(pvs_records)) == 31

    # Test vg method
    assert pvs_records.vg('data1') == [{
        'LVM2_PV_MDA_USED_COUNT': '1',
        'LVM2_PV_UUID': 'dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN',
        'LVM2_DEV_SIZE': '3.64t', 'Fmt': 'lvm2', 'LVM2_PV_MDA_FREE': '0',
        'LVM2_PV_EXPORTED': '', 'LVM2_PV_SIZE': '3.64t',
        'LVM2_PV_PE_ALLOC_COUNT': '953852', 'LVM2_PV_TAGS': '',
        'PFree': '4.00m', 'LVM2_PV_ATTR': 'a--',
        'PV_UUID': 'dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN', 'PV': '/dev/sdb1',
        'LVM2_PV_NAME': '/dev/sdb1', 'Missing': '', '1st_PE': '256.00k',
        'LVM2_PV_MDA_COUNT': '1', 'LVM2_PV_FREE': '4.00m',
        'LVM2_PV_ALLOCATABLE': 'allocatable', 'BA_start': '0',
        'LVM2_PV_MDA_SIZE': '252.00k', 'Exported': '', 'PE': '953853',
        'PV_Tags': '', 'LVM2_PV_EXT_VSN': '2', 'LVM2_PV_MINOR': '17',
        'Alloc': '953852', 'Attr': 'a--', 'VG': 'data1',
        'LVM2_PE_START': '256.00k', 'LVM2_PV_FMT': 'lvm2',
        'DevSize': '3.64t', 'PSize': '3.64t', 'LVM2_PV_BA_START': '0',
        'Used': '3.64t', 'LVM2_VG_NAME': 'data1', 'PMdaSize': '252.00k',
        'LVM2_PV_PE_COUNT': '953853', 'LVM2_PV_BA_SIZE': '0',
        'LVM2_PV_IN_USE': 'used', 'LVM2_PV_USED': '3.64t', '#PMda': '1',
        'PMdaFree': '0', 'Allocatable': 'allocatable', 'BA_size': '0',
        'LVM2_PV_MAJOR': '8', '#PMdaUse': '1', 'LVM2_PV_MISSING': '',
        'LVM2_PV_DUPLICATE': '',
        'PV_KEY': '/dev/sdb1+dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN'
    }]


def test_pvs_dup():
    pvs_records = Pvs(context_wrap(PVS_INFO_DUP))
    assert len(list(pvs_records)) == 2
    pv_keys = set([pv['PV_KEY'] for pv in pvs_records])
    assert pv_keys == set([
        'unknown device+mn5KxB-YKlY-u4hK-zuZJ-Ia6r-3dTg-8IDjsM',
        'unknown device+V4xZ9b-FXOz-CrRA-Eu2e-8iOS-9EDF-YZYftK',
    ])


def test_pvs_headings():
    def check(pvs_records):
        assert len(pvs_records.data) == 9
        for k, v in PVS_HEADINGS_6.items():
            assert pvs_records[6][k] == v
        assert pvs_records[6]['Missing'] is None

    pvs_records = PvsHeadings(context_wrap(PVS_HEADINGS))
    check(pvs_records)
    pvs_records = PvsHeadings(context_wrap(FD_LEAK_HEADER + PVS_HEADINGS))
    check(pvs_records)

    # Test vg method
    fedora_pvs = pvs_records.vg('fedora')
    assert len(fedora_pvs) == 1
    assert compare_partial_dicts(fedora_pvs[0], {
        'PV': '/dev/mapper/luks-7430952e-7101-4716-9b46-786ce4684f8d',
        'VG': 'fedora', 'Fmt': 'lvm2', 'Attr': 'a--', 'PSize': '476.45g',
        'PFree': '4.00m', 'DevSize': '476.45g',
        'PV_UUID': 'FPLCRf-d918-LVL7-6e3d-n3ED-aiZv-EesuzY', 'PMdaFree': '0',
        'PMdaSize': '1020.00k', '#PMda': '1', '#PMdaUse': '1', 'PE': '121970',
        'PV_KEY': '/dev/mapper/luks-7430952e-7101-4716-9b46-786ce4684f8d+FPLCRf-d918-LVL7-6e3d-n3ED-aiZv-EesuzY'
    })



import doctest
import pytest

from insights.parsers import ps, ParseException
from insights.tests import context_wrap
from insights.util import keys_in

PsAuxww_TEST_DOC = """
 USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
 root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
 root      1661  0.0  0.0 126252  1392 ?        Ss   May31   0:04 /usr/sbin/crond -n
 root      1691  0.0  0.0  42688   172 ?        Ss   May31   0:00 /usr/sbin/rpc.mountd
 root      1821  0.0  0.0      0     0 ?        Z    May31   0:29 [kondemand/0]
 root      1864  0.0  0.0  18244   668 ?        Ss   May31   0:05 /usr/sbin/irqbalance --foreground
 user1    20160  0.0  0.0 108472  1896 pts/3    Ss   10:09   0:00 /bin/bash
 root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 /usr/sbin/dhclient enp0s25
 root     20457  0.0  0.0   9120   832 ?        Ss   10:09   0:00 /bin/bash
"""

PsEf_TEST_DOC = """
UID         PID   PPID  C STIME TTY          TIME CMD
root          1      0  0 03:53 ?        00:00:06 /usr/lib/systemd/systemd --system --deserialize 15
root          2      0  0 03:53 ?        00:00:00 [kthreadd]
root       1803      1  5 03:54 ?        00:55:22 /usr/bin/openshift start master --config=/etc/origin/master/master-config.yaml --loglevel
root       1969      1  3 03:54 ?        00:33:51 /usr/bin/openshift start node --config=/etc/origin/node/node-config.yaml --loglevel=2
root       1995      1  0 03:54 ?        00:02:06 /usr/libexec/docker/rhel-push-plugin
root       2078   1969  0 03:54 ?        00:00:00 journalctl -k -f
root       7201      1  0 03:59 ?        00:00:00 /usr/bin/python /usr/libexec/rhsmd
root     111434      1  0 22:32 ?        00:00:00 nginx: master process /usr/sbin/nginx -c /etc/nginx/nginx.conf
nginx    111435 111434  0 22:32 ?        00:00:00 nginx: worker process
"""

PsEo_TEST_DOC = """
  PID  PPID COMMAND
    1     0 systemd
    2     0 kthreadd
    3     2 ksoftirqd/0
 2416     1 auditd
 2419  2416 audispd
 2421  2419 sedispatch
 2892     1 NetworkManager
 3172  2892 dhclient
 3871     1 master
 3886  3871 qmgr
13724  3871 pickup
15663     2 kworker/0:1
16998     2 kworker/0:3
17259     2 kworker/0:0
18294  3357 sshd
"""

PsAlxwww_TEST_DOC = """
F   UID   PID  PPID PRI  NI    VSZ   RSS WCHAN  STAT TTY        TIME COMMAND
4     0     1     0  20   0 128292  6928 ep_pol Ss   ?          0:02 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
1     0     2     0  20   0      0     0 kthrea S    ?          0:00 [kthreadd]
1     0     3     2  20   0      0     0 smpboo S    ?          0:00 [ksoftirqd/0]
5     0     4     2  20   0      0     0 worker S    ?          0:00 [kworker/0:0]
1     0     5     2   0 -20      0     0 worker S<   ?          0:00 [kworker/0:0H]
1     0     6     2  20   0      0     0 worker S    ?          0:00 [kworker/u4:0]
1     0     7     2 -100  -      0     0 smpboo S    ?          0:00 [migration/0]
1     0     8     2  20   0      0     0 rcu_gp S    ?          0:00 [rcu_bh]
"""

PsEoCmd_TEST_DOC = """
  PID COMMAND
    1 /usr/lib/systemd/systemd
    2 [kthreadd]
    11 /usr/bin/python3
    12 [kworker/u16:0-kcryptd/253:0]
"""


def test_doc_examples():
    env = {
        'ps': ps.PsAuxww(context_wrap(PsAuxww_TEST_DOC)),
        'ps_auxww': ps.PsAuxww(context_wrap(PsAuxww_TEST_DOC)),
        'ps_ef': ps.PsEf(context_wrap(PsEf_TEST_DOC)),
        'ps_eo': ps.PsEo(context_wrap(PsEo_TEST_DOC, strip=False)),
        'ps_alxwww': ps.PsAlxwww(context_wrap(PsAlxwww_TEST_DOC)),
        'ps_eo_cmd': ps.PsEoCmd(context_wrap(PsEoCmd_TEST_DOC))
    }
    failed, total = doctest.testmod(ps, globs=env)
    assert failed == 0


PsAuxww_TEST = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0 193892  5000 ?        Ss   Oct23   1:40 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
root         2  0.0  0.0      0     0 ?        S    Oct23   0:00 [kthreadd]
root       781  0.0  0.0 214308 10472 ?        Ss   Oct23   1:10 /usr/lib/systemd/systemd-journald
root      1111  0.0  0.0  24316  1400 ?        Ss   Oct23   0:00 /usr/sbin/smartd -n -q never
polkitd   1112  0.0  0.0 664820 14536 ?        Ssl  Oct23   2:35 /usr/lib/polkit-1/polkitd --no-debug
user_01   2713  2.4 19.2 12508144 3077224 ?    Sl   Nov21 533:33 /usr/lib64/firefox/firefox
root      8257  0.0  0.0 112876 14848 ?        S    08:18   0:00 /sbin/dhclient -d -q -sf /usr/libexec/nm-dhcp-helper -pf /var/run/dhclient-enp0s25.pid -lf /var/lib/NetworkManager/dhclient-bdb77870-4bb7-44e4-9f4f-d15ededcb42c-enp0s25.lease -cf /var/lib/NetworkManager/dhclient-enp0s25.conf enp0s25
"""


def test_ps_auxww_from_auxww():
    # test with input from `ps auxww`
    p = ps.PsAuxww(context_wrap(PsAuxww_TEST))
    d = p.data
    assert all('COMMAND' in row for row in d)
    assert keys_in([
        "USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY", "STAT", "START",
        "TIME", "COMMAND"
    ], d[0])
    assert d[0] == {
        '%MEM': '0.0', 'TTY': '?', 'VSZ': '193892', 'PID': '1', '%CPU': '0.0',
        'START': 'Oct23', 'USER': 'root', 'STAT': 'Ss', 'TIME': '1:40', 'RSS': '5000',
        'COMMAND': '/usr/lib/systemd/systemd --switched-root --system --deserialize 22',
        'COMMAND_NAME': 'systemd', 'ARGS': '--switched-root --system --deserialize 22',
    }
    assert d[2]["COMMAND"] == '/usr/lib/systemd/systemd-journald'
    assert d[-2] == {
        'USER': 'user_01', 'PID': '2713', '%CPU': '2.4', '%MEM': '19.2',
        'VSZ': '12508144', 'RSS': '3077224', 'TTY': '?', 'STAT': 'Sl',
        'START': 'Nov21', 'TIME': '533:33', 'COMMAND': '/usr/lib64/firefox/firefox',
        'COMMAND_NAME': 'firefox', 'ARGS': '',
    }
    assert p.fuzzy_match('kthreadd')
    assert '[kthreadd]' in p
    assert 'sshd' not in p
    assert not p.fuzzy_match("sshd")
    assert p.number_occurences("systemd") == 2
    assert p.number_occurences("openshift") != 2
    assert p.cpu_usage('/usr/lib64/firefox/firefox') == '2.4'
    assert p.cpu_usage('firefox') is None
    assert p.cpu_usage('dhclient') is None
    assert p.cpu_usage('java') is None
    assert p.running_pids() == ['1', '2', '781', '1111', '1112', '2713', '8257']

    # Test __iter__
    assert 13718356 == sum(int(proc['VSZ']) for proc in p)


PsEf_TEST = """
UID         PID   PPID  C STIME TTY          TIME CMD
root          1      0  0 03:53 ?        00:00:06 /usr/lib/systemd/systemd --system --deserialize 15
root          2      0  0 03:53 ?        00:00:00 [kthreadd]
root       1803      1  5 03:54 ?        00:55:22 /usr/bin/openshift start master --config=/etc/origin/master/master-config.yaml --loglevel
root       1969      1  3 03:54 ?        00:33:51 /usr/bin/openshift start node --config=/etc/origin/node/node-config.yaml --loglevel=2
root       1995      1  0 03:54 ?        00:02:06 /usr/libexec/docker/rhel-push-plugin
root       2078   1969  0 03:54 ?        00:00:00 journalctl -k -f
root       7201      1  0 03:59 ?        00:00:00 /usr/bin/python /usr/libexec/rhsmd
root     111434      1  0 22:32 ?        00:00:00 nginx: master process /usr/sbin/nginx -c /etc/nginx/nginx.conf
nginx    111435 111434  0 22:32 ?        00:00:00 nginx: worker process
"""


def test_ps_ef_from_ef():
    # test with input from `ps -ef`
    p = ps.PsEf(context_wrap(PsEf_TEST))
    d = p.data
    assert all('CMD' in row for row in d)
    assert keys_in([
        "UID", "PID", "PPID", "C", "STIME", "TTY", "TIME", "CMD"
    ], d[0])
    assert d[0] == {
        'UID': 'root', 'TTY': '?', 'PID': '1', 'PPID': '0',
        'TIME': '00:00:06', 'STIME': '03:53', 'C': '0',
        'CMD': '/usr/lib/systemd/systemd --system --deserialize 15',
        'COMMAND_NAME': 'systemd', 'ARGS': '--system --deserialize 15',
    }
    assert d[4]["CMD"] == '/usr/libexec/docker/rhel-push-plugin'
    assert d[-5] == {
        'TTY': '?', 'ARGS': '', 'UID': 'root',
        'CMD': '/usr/libexec/docker/rhel-push-plugin',
        'PID': '1995', 'C': '0', 'STIME': '03:54', 'TIME': '00:02:06',
        'PPID': '1', 'COMMAND_NAME': 'rhel-push-plugin'
    }
    assert p.fuzzy_match('kthreadd')
    assert p.number_occurences("systemd") != 2
    assert p.number_occurences("openshift") == 2
    assert p.parent_pid("111435") == ["111434", "nginx: master process /usr/sbin/nginx -c /etc/nginx/nginx.conf"]
    assert '[kthreadd]' in p
    assert 'sshd' not in p
    assert not p.fuzzy_match("sshd")
    assert p.running_pids() == ['1', '2', '1803', '1969', '1995', '2078', '7201', '111434', '111435']
    assert p.users("nginx: worker process") == {'nginx': ['111435']}


PsAuxcww_TEST = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      1821  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root      1864  0.0  0.0  18244   668 ?        Ss   May31   0:05 irqbalance
user1    20160  0.0  0.0 108472  1896 pts/3    Ss   10:09   0:00 bash
user2    20161  0.0  0.0 108472  1896 pts/4    Ss   10:09   0:00 bash
user2    20164  0.0  0.0 108472  1896 pts/5    Ss   10:10   0:00 bash
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
qemu     22673  0.6 10.7 1618556 840452 ?      Sl   11:38   1:31 qemu-kvm
vdsm     27323 98.0 11.3  9120    987 ?        Ss   10.01   1:31 vdsm
""".strip()

PsAuxcww_BAD = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0
""".strip()

PsAuxcww_BAD_1 = """
root         1  0.0  0.0
""".strip()

PsAuxcww_BAD_2 = ""


def test_ps_auxww_from_auxcww():
    # test with input from `ps auxcww`
    p = ps.PsAuxww(context_wrap(PsAuxcww_TEST))
    d = p.data
    assert all('COMMAND' in row for row in d)
    assert keys_in([
        "USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY", "STAT", "START",
        "TIME", "COMMAND"
    ], d[0])
    assert d[0] == {
        '%MEM': '0.0', 'TTY': '?', 'VSZ': '19356', 'PID': '1', '%CPU': '0.0',
        'START': 'May31', 'COMMAND': 'init', 'COMMAND_NAME': 'init', 'USER': 'root', 'STAT': 'Ss',
        'TIME': '0:01', 'RSS': '1544', 'ARGS': '',
    }
    assert d[2]["COMMAND"] == 'irqbalance'
    assert d[-2]["COMMAND"] == 'qemu-kvm'
    assert p.fuzzy_match('irqbal')
    assert p.number_occurences("bash") == 3
    assert p.number_occurences("qemu-kvm") != 2
    assert 'dhclient' in p
    assert 'sshd' not in p
    assert not p.fuzzy_match("sshd")
    assert p.cpu_usage('dhclient') == '0.0'
    assert p.cpu_usage('java') is None

    assert p.users('bash') == {'user1': ['20160'], 'user2': ['20161', '20164']}


PS_AUXWWW = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  21452  1536 ?        Ss   Mar09   0:01 /sbin/init
root         2  0.0  0.0      0     0 ?        S    Mar09   0:00 [kthreadd]
root       501  0.0  0.0  11608  1684 ?        S<s  Mar09   0:00 /sbin/udevd -d
root      1319  0.0  0.0 326992  1684 ?        Sl   Mar09   0:00 /sbin/rsyslogd -i /var/run/syslogd.pid -c 5
dbus      1332  0.0  0.0  33740  1028 ?        Ssl  Mar09   0:00 dbus-daemon --system
root      1533  0.0  0.0   4060   584 tty4     Ss+  Mar09   0:00 /sbin/mingetty /dev/tty4
root      1606  0.0  0.0  83204  2432 ?        Ss   Mar09   0:00 login -- root
root      1610  0.0  0.0 108424  2104 tty1     Ss   Mar09   0:00 -bash
root      2573  1.6  1.0 342188 42020 tty1     S+   17:46   0:01 /usr/bin/python /usr/sbin/sosreport
"""


def test_ps_auxww_from_auxwww():
    # test with input from `ps auxwww`
    p = ps.PsAuxww(context_wrap(PS_AUXWWW))
    d = p.data
    assert all('COMMAND' in row for row in d)
    assert keys_in([
        "USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY", "STAT", "START",
        "TIME", "COMMAND"
    ], d[0])
    assert d[0] == {
        'USER': 'root', 'PID': '1', '%CPU': '0.0', '%MEM': '0.0',
        'VSZ': '21452', 'RSS': '1536', 'TTY': '?', 'STAT': 'Ss',
        'START': 'Mar09', 'TIME': '0:01', 'COMMAND': '/sbin/init',
        'COMMAND_NAME': 'init', 'ARGS': '',
    }
    assert p.fuzzy_match('kthread')
    assert '-bash' in p
    assert '/sbin/init' in p
    assert 'sshd' not in p
    assert 'kthread' not in p
    assert not p.fuzzy_match("sshd")
    assert p.number_occurences("python") == 1
    assert p.number_occurences("sshd") != 2
    assert p.fuzzy_match('python')

    assert p.search() == []
    assert p.search(COMMAND__contains='rsyslogd') == [p.data[3]]
    assert p.search(USER='root', COMMAND__contains='kthread') == [p.data[1]]
    assert p.search(TTY='tty1') == [p.data[7], p.data[8]]
    assert p.search(STAT__contains='Z') == []


PsAux_TEST = """
USER       PID %CPU %MEM     VSZ    RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0   19356   1544 ?        Ss   May31   0:01 /sbin/init
root      1821  0.0  0.0       0      0 ?        S    May31   0:25 [kondemand/0]
root      1864  0.0  0.0   18244    668 ?        Ss   May31   0:05 irqbalance --pid=/var/run/irqbalance.pid
user1    20160  0.0  0.0  108472   1896 pts/3    Ss   10:09   0:00 bash
root     20357  0.0  0.0    9120    760 ?        Ss   10:09   0:00 /sbin/dhclient -1 -q -lf /var/lib/dhclient/dhclient-extbr0.leases -pf /var/run/dhclient-extbr0.pid extbr0
qemu     22673  0.8 10.2 1618556 805636 ?        Sl   11:38   1:07 /usr/libexec/qemu-kvm -name rhel7 -S -M rhel6.5.0 -enable-kvm -m 1024 -smp 2,sockets=2,cores=1,threads=1 -uuid 13798ffc-bc1e-d437-4f3f-2e0fa6c923ad
tomcat    3662  1.0  5.7 2311488  58236 ?        Ssl  07:28   0:01 /usr/lib/jvm/jre/bin/java -classpath /usr/share/tomcat/bin/bootstrap.jar:/usr/share/tomcat/bin/tomcat-juli.jar:/usr/share/java/commons-daemon.jar -Dcatalina.base=/usr/share/tomcat -Dcatalina.home=/usr/share/tomcat -Djava.endorsed.dirs= -Djava.io.tmpdir=/var/cache/tomcat/temp -Djava.util.logging.config.file=/usr/share/tomcat/conf/logging.properties -Djava.util.logging.manager=org.apache.juli.ClassLoaderLogManager org.apache.catalina.startup.Bootstrap start
""".strip()


def test_ps_auxww_from_aux():
    # test with input from `ps aux`
    p = ps.PsAuxww(context_wrap(PsAux_TEST))
    d = p.data
    assert all('COMMAND' in row for row in d)
    assert keys_in([
        "USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY", "STAT", "START",
        "TIME", "COMMAND"
    ], d[0])
    assert d[0] == {
        '%MEM': '0.0', 'TTY': '?', 'VSZ': '19356', 'PID': '1', '%CPU': '0.0',
        'START': 'May31', 'COMMAND': '/sbin/init', 'COMMAND_NAME': 'init', 'USER': 'root',
        'STAT': 'Ss', 'TIME': '0:01', 'RSS': '1544', 'ARGS': ''
    }
    assert p.fuzzy_match('irqbal')
    assert 'bash' in p
    assert '/sbin/init' in p
    assert 'sshd' not in p
    assert 'kondemand' not in p
    assert not p.fuzzy_match("sshd")
    assert p.number_occurences("systemd") != 2
    assert p.number_occurences("bash") == 1
    assert p.fuzzy_match('kondemand')

    assert p.search() == []
    assert p.search(COMMAND__contains='java') == [p.data[6]]
    assert p.search(USER='root', COMMAND__contains='kondemand') == [p.data[1]]
    assert p.search(TTY='pts/3') == [p.data[3]]
    assert p.search(STAT__contains='Z') == []


PS_AUXWW_WITH_LATE_HEADER = '''
your 131072x1 screen size is bogus. expect trouble
USER        PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root          1  0.0  0.1 193704  5088 ?        Ss   Jan10   6:52 /usr/lib/systemd/systemd --switched-root --system --deserialize 21
root          2  0.0  0.0      0     0 ?        S    Jan10   0:02 [kthreadd]
root          3  0.0  0.0      0     0 ?        S    Jan10   0:29 [ksoftirqd/0]
root          5  0.0  0.0      0     0 ?        S<   Jan10   0:00 [kworker/0:0H]
root          7  0.0  0.0      0     0 ?        S    Jan10   0:00 [migration/0]
'''


def test_ps_auxww_with_late_header():
    p = ps.PsAuxww(context_wrap(PS_AUXWW_WITH_LATE_HEADER))
    assert '[kthreadd]' in p
    assert p.data[0]['COMMAND'] == '/usr/lib/systemd/systemd --switched-root --system --deserialize 21'


Ps_BAD = """
/bin/ps: command or file not found
"""


def test_ps_auxww_with_bad_input():
    # test with bad input
    d1 = ps.PsAuxww(context_wrap(PsAuxcww_BAD))
    assert 'test' not in d1

    with pytest.raises(ParseException) as exc:
        d2 = ps.PsAuxww(context_wrap(Ps_BAD))
        assert d2 is None
    assert 'PsAuxww: Cannot find ps header line containing' in str(exc)


PS_EO_NORMAL = """
  PID  PPID COMMAND
    1     0 systemd
    2     0 kthreadd
    3     2 ksoftirqd/0
    5     2 kworker/0:0H
    6     2 kworker/u2:0
 2416     1 auditd
 2419  2416 audispd
 2421  2419 sedispatch
 2892     1 NetworkManager
 3172  2892 dhclient
 3871     1 master
 3886  3871 qmgr
13724  3871 pickup
15663     2 kworker/0:1
16998     2 kworker/0:3
17259     2 kworker/0:0
18294  3357 sshd
18302 18294 sshd
18303 18302 bash
18338 18303 sudo
18346 18338 su
18347 18346 bash
18379 18347 ps
"""


def test_ps_eo():
    p = ps.PsEo(context_wrap(PS_EO_NORMAL, strip=False))
    assert p is not None
    assert len(p.pid_info) == 23
    assert '15663' in p.pid_info
    assert p.pid_info['2416'] == {
        'PID': '2416', 'PPID': '1', 'COMMAND': 'auditd', 'COMMAND_NAME': 'auditd', 'ARGS': ''
    }
    assert p.pid_info['18379'] == {
        'PID': '18379', 'PPID': '18347', 'COMMAND': 'ps', 'COMMAND_NAME': 'ps', 'ARGS': ''
    }
    assert p.children('18347') == [
        {'PID': '18379', 'PPID': '18347', 'COMMAND': 'ps', 'COMMAND_NAME': 'ps', 'ARGS': ''}
    ]
    assert len(p.children('2')) == 6


def test_ps_eo_stripped():
    p = ps.PsEo(context_wrap(PS_EO_NORMAL, strip=True))
    assert p is not None
    assert len(p.pid_info) == 23


PS_ALXWWW_DATA = """
F   UID   PID  PPID PRI  NI    VSZ   RSS WCHAN  STAT TTY        TIME COMMAND
4     0     1     0  20   0 128292  6944 ep_pol Ss   ?          0:02 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
1     0     2     0  20   0      0     0 kthrea S    ?          0:00 [kthreadd]
1     0     3     2  20   0      0     0 smpboo S    ?          0:00 [ksoftirqd/0]
5     0     4     2  20   0      0     0 worker S    ?          0:00 [kworker/0:0]
1     0     5     2   0 -20      0     0 worker S<   ?          0:00 [kworker/0:0H]
4     0  1585     1  20   0  39336  3872 ep_pol Ss   ?          0:00 /usr/lib/systemd/systemd-journald
5     0  2964     1  16  -4  55520   900 ep_pol S<sl ?          0:00 /sbin/auditd
4     0  2966  2964  12  -8  84552   896 futex_ S<sl ?          0:00 /sbin/audispd
4     0  2968  2966  16  -4  55628  1404 unix_s S<   ?          0:00 /usr/sbin/sedispatch
4    81  3000     1  20   0  69800  3580 ep_pol Ssl  ?          0:00 /usr/bin/dbus-daemon --system --address=systemd: --nofork --nopidfile --systemd-activation
"""


def test_ps_alxwww():
    p = ps.PsAlxwww(context_wrap(PS_ALXWWW_DATA))
    assert p is not None
    assert len(p.data) > 0
    assert set(p.data[0].keys()) == set([
        'F', 'UID', 'PID', 'PPID', 'PRI', 'NI', 'VSZ', 'RSS', 'WCHAN',
        'STAT', 'TTY', 'TIME', 'COMMAND', 'COMMAND_NAME', 'ARGS'
    ])
    assert 'auditd' in p.cmd_names
    assert 'dbus-daemon' in p.cmd_names
    assert '/usr/bin/dbus-daemon --system --address=systemd: --nofork --nopidfile --systemd-activation' in p.running
    assert '/usr/lib/systemd/systemd --switched-root --system --deserialize 22' in p
    assert p.running_pids() == ['1', '2', '3', '4', '5', '1585', '2964', '2966', '2968', '3000']
    dbus_proc = next((proc for proc in p.data if proc['COMMAND_NAME'] == 'dbus-daemon'), None)
    assert dbus_proc is not None
    assert dbus_proc['COMMAND_NAME'] == 'dbus-daemon'
    assert dbus_proc['UID'] == '81'
    assert dbus_proc['ARGS'] == '--system --address=systemd: --nofork --nopidfile --systemd-activation'


PS_EO_CMD_NORMAL = """
    PID COMMAND
      1 /usr/lib/systemd/systemd
      2 [kthreadd]
      3 [rcu_gp]
  93831 qmgr
  93838 tlsmgr
1221279 /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.242.b08-4.el8.x86_64/jre/bin/java
1221774 /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.242.b08-4.el8.x86_64/jre/bin/java
"""


def test_ps_eo_cmd():
    p = ps.PsEoCmd(context_wrap(PS_EO_CMD_NORMAL, strip=False))
    assert p is not None
    assert len(p.running_pids()) == 7
    assert '93838' in p.pid_info
    assert p.pid_info['1221279'] == {
        'PID': '1221279', 'COMMAND': '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.242.b08-4.el8.x86_64/jre/bin/java', 'COMMAND_NAME': 'java', 'ARGS': ''
    }
    assert p.pid_info['3'] == {
        'PID': '3', 'COMMAND': '[rcu_gp]', 'COMMAND_NAME': '[rcu_gp]', 'ARGS': ''
    }
    assert p.pid_info['93831'] == {
        'PID': '93831', 'COMMAND': 'qmgr', 'COMMAND_NAME': 'qmgr', 'ARGS': ''
    }


def test_ps_eo_cmd_stripped():
    p = ps.PsEo(context_wrap(PS_EO_CMD_NORMAL, strip=True))
    assert p is not None
    assert len(p.running_pids()) == 7



"""
``/sys/module`` System Module Information
=========================================

A parser to parse the system module information.

Parsers included in this module are:

DMModUseBlkMq - file ``/sys/module/dm_mod/parameters/use_blk_mq``
------------------------------------------------------------------
LpfcMaxLUNs - file ``/sys/module/lpfc/parameters/lpfc_max_luns``
----------------------------------------------------------------
Ql2xMaxLUN - file ``/sys/module/qla2xxx/parameters/ql2xmaxlun``
---------------------------------------------------------------
SCSIModMaxReportLUNs - file ``/sys/module/scsi_mod/parameters/max_report_luns``
-------------------------------------------------------------------------------
SCSIModUseBlkMq - file ``/sys/module/scsi_mod/parameters/use_blk_mq``
---------------------------------------------------------------------
VHostNetZeroCopyTx - file ``/sys/module/vhost_net/parameters/experimental_zcopytx``
-----------------------------------------------------------------------------------
"""
from insights import parser, Parser
from insights.parsers import SkipException
from insights.specs import Specs


class XModUseBlkMq(Parser):
    """
    Parse for file `/sys/module/{dm_mod,scsi_mod}/parameters/use_blk_mq`.
    File content shows if use_blk_mq parameter is on.

    Sample Content::

        Y

    Raises:
        SkipException: When nothing need to parse.

    Attributes:
        val(str): Raw data of the content.
    """

    def parse_content(self, content):
        if not content or len(content) != 1:
            raise SkipException()
        self.val = content[0].strip()

    @property
    def is_on(self):
        """
        Returns:
            (bool): True for on, False for off.

        Raises:
            ValueError: When tell is_on for unknown cases.
        """
        if self.val in ['Y', '1']:
            return True
        if self.val in ['N', '0']:
            return False
        raise ValueError("Unexpected value {0}, please get raw data from attribute 'val' and tell is_on by yourself.".format(self.val))


class MaxLUNs(Parser):
    """
    Parse for file `/sys/module/{scsi_mod, lpfc, ...}/parameters/{max_report_luns, lpfc_max_luns, ...}`.
    File content shows the maximum LUN value currently supported.

    Sample Content::

        512

    Raises:
        SkipException: When content is empty or no parse-able content.

    Attributes:
        val(int): Convert the raw data of the content to int.
    """

    def parse_content(self, content):
        if not content or len(content) != 1:
            raise SkipException()
        if not content[0].strip('').isdigit():
            raise ValueError("Unexpected content: {0}".format(content[0]))
        self.val = int(content[0].strip())


@parser(Specs.dm_mod_use_blk_mq)
class DMModUseBlkMq(XModUseBlkMq):
    """
    This file `/sys/module/dm_mod/parameters/use_blk_mq` shows if use_blk_mq
    parameter is on.

    Examples::

        >>> dm_mod_use_blk_mq.val
        'Y'
        >>> dm_mod_use_blk_mq.is_on
        True
    """
    pass


@parser(Specs.scsi_mod_use_blk_mq)
class SCSIModUseBlkMq(XModUseBlkMq):
    """
    This file `/sys/module/scsi_mod/parameters/use_blk_mq` shows if use_blk_mq
    parameter is on.

    Examples::

        >>> scsi_mod_use_blk_mq.val
        'N'
        >>> scsi_mod_use_blk_mq.is_on
        False
    """
    pass


@parser(Specs.vhost_net_zero_copy_tx)
class VHostNetZeroCopyTx(XModUseBlkMq):
    """This file `/sys/module/vhost_net/parameters/experimental_zcopytx` shows if
    vhost_net's zero-copy tx parameter is enabled or not.

    Examples::

        >>> vhost_net_zero_copy_tx.val
        '0'
        >>> vhost_net_zero_copy_tx.is_on
        False

    """
    pass


@parser(Specs.lpfc_max_luns)
class LpfcMaxLUNs(MaxLUNs):
    """
    This file `/sys/module/lpfc/parameters/lpfc_max_luns` shows the max LUN number
    supported by lpfc driver.

    Examples:

        >>> lpfc_max_luns.val
        512
    """
    pass


@parser(Specs.ql2xmaxlun)
class Ql2xMaxLUN(MaxLUNs):
    """
    This file `/sys/module/qla2xxx/parameters/ql2xmaxlun` shows the max LUN number
    supported by qla2xxxx driver.

    Examples:

        >>> ql2xmaxlun.val
        512
    """
    pass


@parser(Specs.scsi_mod_max_report_luns)
class SCSIModMaxReportLUNs(MaxLUNs):
    """
    This file `/sys/module/scsi_mod/parameters/max_report_luns` shows the max LUN number
    supported by OS.

    Examples:

        >>> scsi_mod_max_report_luns.val
        512
    """
    pass



"""
SystemctlShow - command ``systemctl show``
==========================================

Parsers included in this module are:

SystemctlShowServiceAll - command ``systemctl show *.service``
--------------------------------------------------------------
SystemctlShowTarget - command ``systemctl show *.target``
-------------------------------------------------------------------
"""
from insights import parser, CommandParser
from insights.parsers import split_kv_pairs, SkipException, ParseException
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.systemctl_show_all_services)
class SystemctlShowServiceAll(CommandParser, dict):
    """
    Class for parsing ``systemctl show *.service`` command output.
    Empty properties are suppressed.

    Sample Input::

        Id=postfix.service
        Names=postfix.service
        TimeoutStartUSec=1min 30s
        LimitNOFILE=65536
        LimitMEMLOCK=
        LimitLOCKS=18446744073709551615

        Id=postgresql.service
        Names=postgresql.service
        Requires=basic.target
        LimitMSGQUEUE=819200
        LimitNICE=0

    Sample Output::

        {
            "postfix.service": {
                "Id"               : "postfix.service",
                "Names"            : "postfix.service",
                "LimitNOFILE"      : "65536",
                "TimeoutStartUSec" : "1min 30s",
                "LimitLOCKS"       : "18446744073709551615",
            },
            "postgresql.service": {
                "Id"               : "postgresql.service",
                "Names"            : "postgresql.service",
                "Requires"         : "basic.target",
                "LimitMSGQUEUE"    : "819200",
                "LimitNICE"        : "0",
            }
        }

    Examples:
        >>> 'postfix' in systemctl_show_all  # ".service" is needed
        False
        >>> 'postfix.service' in systemctl_show_all
        True
        >>> systemctl_show_all['postfix.service']['Id']
        'postfix.service'
        >>> 'LimitMEMLOCK' in systemctl_show_all['postfix.service']
        False
        >>> systemctl_show_all['postfix.service']['LimitLOCKS']
        '18446744073709551615'
        >>> 'postgresql.service' in systemctl_show_all
        True
        >>> systemctl_show_all['postgresql.service']['LimitNICE']
        '0'

    Raises:
        SkipException: When nothing needs to parse
        ParseException: When something cannot be parsed
    """

    def parse_content(self, content):
        if not content:
            raise SkipException

        sidx = 0
        idx_list = []
        for i, l in enumerate(content):
            if l.strip() == '':
                idx_list.append((sidx, i))
                sidx = i + 1
        idx_list.append((sidx, len(content)))
        for s, e in idx_list:
            data = split_kv_pairs(content[s:e], use_partition=False)
            name = data.get('Names', data.get('Id'))
            if not name:
                raise ParseException('"Names" or "Id" not found!')
            self[name] = dict((k, v) for k, v in data.items() if v)

        if len(self) == 0:
            raise SkipException


@parser(Specs.systemctl_show_target)
class SystemctlShowTarget(SystemctlShowServiceAll):
    """
    Class for parsing ``systemctl show *.target`` command output.
    Empty properties are suppressed.

    This class is inherited from :py:class:`SystemctlShowServiceAll`.

    Sample Input::

        Id=network.target
        Names=network.target
        WantedBy=NetworkManager.service
        Conflicts=shutdown.target
        Before=tuned.service network-online.target rhsmcertd.service kdump.service httpd.service rsyslog.service rc-local.service insights-client.timer insights-client.service sshd.service postfix.service
        After=firewalld.service network-pre.target network.service NetworkManager.service
        Documentation=man:systemd.special(7) http://www.freedesktop.org/wiki/Software/systemd/NetworkTarget
        Description=Network
        LoadState=loaded
        ActiveState=active
        SubState=active
        FragmentPath=/usr/lib/systemd/system/network.target
        UnitFileState=static
        UnitFilePreset=disabled
        InactiveExitTimestamp=Tue 2020-02-25 10:39:46 GMT
        InactiveExitTimestampMonotonic=15332468
        ActiveEnterTimestamp=Tue 2020-02-25 10:39:46 GMT
        ActiveEnterTimestampMonotonic=15332468
        ActiveExitTimestampMonotonic=0
        InactiveEnterTimestampMonotonic=0
        CanStart=no

    Sample Output::

        {'network.target': {'ActiveEnterTimestamp': 'Tue 2020-02-25 10:39:46 GMT',
                            'ActiveEnterTimestampMonotonic': '15332468',
                            'ActiveExitTimestampMonotonic': '0',
                            'ActiveState': 'active',
                            'After': 'firewalld.service network-pre.target '
                                     'network.service NetworkManager.service',
                            'Before': 'tuned.service network-online.target '
                                      'rhsmcertd.service kdump.service httpd.service '
                                      'rsyslog.service rc-local.service '
                                      'insights-client.timer insights-client.service '
                                      'sshd.service postfix.service',
                            'CanStart': 'no',
                            'Conflicts': 'shutdown.target',
                            'Description': 'Network',
                            'Documentation': 'man:systemd.special(7) '
                                             'http://www.freedesktop.org/wiki/Software/systemd/NetworkTarget',
                            'FragmentPath': '/usr/lib/systemd/system/network.target',
                            'Id': 'network.target',
                            'InactiveEnterTimestampMonotonic': '0',
                            'InactiveExitTimestamp': 'Tue 2020-02-25 10:39:46 GMT',
                            'InactiveExitTimestampMonotonic': '15332468',
                            'LoadState': 'loaded',
                            'Names': 'network.target',
                            'SubState': 'active',
                            'UnitFilePreset': 'disabled',
                            'UnitFileState': 'static',
                            'WantedBy': 'NetworkManager.service'})

    Examples:
        >>> 'network.target' in systemctl_show_target
        True
        >>> systemctl_show_target.get('network.target').get('WantedBy', None)
        'NetworkManager.service'
        >>> systemctl_show_target.get('network.target').get('RequiredBy', None)

    Raises:
        SkipException: When nothing needs to parse
        ParseException: When something cannot be parsed
    """
    pass


class SystemctlShow(CommandParser, dict):
    """
    .. warning::
        This class is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for parsing ``systemctl show <Service_Name>`` command output.
    Empty properties are suppressed.

    Sample Input::

        TimeoutStartUSec=1min 30s
        LimitNOFILE=65536
        LimitMEMLOCK=
        LimitLOCKS=18446744073709551615

    Sample Output::

        {"LimitNOFILE"     : "65536",
        "TimeoutStartUSec" : "1min 30s",
        "LimitLOCKS"       : "18446744073709551615"}

    In CMD's output, empty properties are suppressed by default.
    """
    def __init__(self, *args, **kwargs):
        deprecated(SystemctlShow, "Deprecated. Use 'SystemctlShowServiceAll' instead.")
        super(SystemctlShow, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        data = split_kv_pairs(content, use_partition=False)
        """Remove empty key"""
        self.update(dict((k, v) for k, v in data.items() if not v == ''))

    @property
    def data(self):
        return self


@parser(Specs.systemctl_cinder_volume)
class SystemctlShowCinderVolume(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show openstack-cinder-volume``.

    Typical output of ``/bin/systemctl show openstack-cinder-volume`` command is::

        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        WatchdogUSec=0
        LimitCORE=18446744073709551615
        LimitRSS=18446744073709551615
        LimitNOFILE=65536
        LimitAS=18446744073709551615
        LimitNPROC=63391
        Transient=no
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_cinder_volume["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_mariadb)
class SystemctlShowMariaDB(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show mariadb``.

    Typical output of ``/bin/systemctl show mariadb`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStopUSec=5min
        ExecStartPre={ path=/usr/libexec/mariadb-prepare-db-dir ; argv[]=/usr/libexec/mariadb-prepare-db-dir %n ; ignore_errors=no ; start_time=[Mon 2017-05-22 06:49:01 EDT] ; stop_time=[Mon 2017-05-22 06:49:02 EDT] ; pid=1946 ; code=exited ; status=0 }
        ExecStart={ path=/usr/bin/mysqld_safe ; argv[]=/usr/bin/mysqld_safe --basedir=/usr ; ignore_errors=no ; start_time=[Mon 2017-05-22 06:49:02 EDT] ; stop_time=[n/a] ; pid=2304 ; code=(null) ; status=0/0 }
        ExecStartPost={ path=/usr/libexec/mariadb-wait-ready ; argv[]=/usr/libexec/mariadb-wait-ready $MAINPID ; ignore_errors=no ; start_time=[Mon 2017-05-22 06:49:02 EDT] ; stop_time=[Mon 2017-05-22 06:49:12 EDT] ; pid=2305 ; code=exited ; status=0 }
        Slice=system.slice
        ControlGroup=/system.slice/mariadb.service
        After=network.target -.mount systemd-journald.socket tmp.mount basic.target syslog.target system.slice
        MemoryCurrent=18446744073709551615
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_mariadb["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_pulp_workers)
class SystemctlShowPulpWorkers(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show pulp_workers``.

    Typical output of ``/bin/systemctl show pulp_workers`` command is::

        Type=oneshot
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=0
        TimeoutStopUSec=1min 30s
        WatchdogUSec=0
        WatchdogTimestampMonotonic=0
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:33 CST
        ExecMainStartTimestampMonotonic=105521850
        ExecMainExitTimestamp=Thu 2018-01-11 14:22:33 CST
        ExecMainExitTimestampMonotonic=105593405
        ExecStart={ path=/usr/libexec/pulp-manage-workers ; argv[]=/usr/libexec/pulp-manage-workers start ; ignore_err
        ExecStop={ path=/usr/libexec/pulp-manage-workers ; argv[]=/usr/libexec/pulp-manage-workers stop ; ignore_error
        Slice=system.slice
        After=systemd-journald.socket system.slice network.target basic.target
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_pulp_workers["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_pulp_resmg)
class SystemctlShowPulpResourceManager(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show pulp_resource_manager``.

    Typical output of ``/bin/systemctl show pulp_resource_manager`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:33 CST
        ExecMainStartTimestampMonotonic=105028117
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2810
        ExecMainCode=0
        ExecMainStatus=0
        ExecStart={ path=/usr/bin/celery ; argv[]=/usr/bin/celery worker -A pulp.server.async.app -n resource_manager@
        Slice=system.slice
        After=basic.target network.target system.slice -.mount systemd-journald.socket
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_pulp_resource_manager["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_pulp_celerybeat)
class SystemctlShowPulpCelerybeat(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show pulp_celerybeat``.

    Typical output of ``/bin/systemctl show pulp_celerybeat`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:32 CST
        ExecMainStartTimestampMonotonic=104261679
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2747
        ExecMainCode=0
        ExecMainStatus=0
        ExecStart={ path=/usr/bin/celery ; argv[]=/usr/bin/celery beat --scheduler=pulp.server.async.scheduler.Schedul
        Slice=system.slice
        After=basic.target network.target system.slice -.mount systemd-journald.socket
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_pulp_celerybeat["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_httpd)
class SystemctlShowHttpd(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show httpd``.

    Typical output of ``/bin/systemctl show httpd`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:32 CST
        ExecMainStartTimestampMonotonic=104261679
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2747
        ExecMainCode=0
        ExecMainStatus=0
        ExecStart={ path=/usr/sbin/httpd ; argv[]=/usr/sbin/httpd $OPTIONS -DFOREGROUND ; ignore_errors=no ; start_time=[Tue 2018-05-15 09:30:08 CST] ; stop_time=[n/a] ; pid=1605 ; code=(null) ; status=0/0 }
        ExecReload={ path=/usr/sbin/httpd ; argv[]=/usr/sbin/httpd $OPTIONS -k graceful ; ignore_errors=no ; start_time=[Wed 2018-05-16 03:07:01 CST] ; stop_time=[Wed 2018-05-16 03:07:01 CST] ; pid=21501 ; code=exited ; status=0 }
        ExecStop={ path=/bin/kill ; argv[]=/bin/kill -WINCH ${MAINPID} ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; status=0/0 }
        Slice=system.slice
        ControlGroup=/system.slice/httpd.service
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_httpd["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_nginx)
class SystemctlShowNginx(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show nginx``.

    Typical output of ``/bin/systemctl show nginx`` command is::

        Type=forking
        Restart=no
        PIDFile=/run/nginx.pid
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=5s
        RuntimeMaxUSec=infinity
        WatchdogUSec=0
        WatchdogTimestampMonotonic=0
        PermissionsStartOnly=no
        RootDirectoryStartOnly=no
        RemainAfterExit=no
        GuessMainPID=yes
        MainPID=0
        ControlPID=0
        FileDescriptorStoreMax=0
        NFileDescriptorStore=0
        StatusErrno=0
        Result=success
        UID=[not set]
        GID=[not set]
        NRestarts=0
        ExecMainStartTimestampMonotonic=0
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=0
        ExecMainCode=0
        ExecMainStatus=0
        ExecStartPre={ path=/usr/bin/rm ; argv[]=/usr/bin/rm -f /run/nginx.pid ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; status=0/0 }
        ExecStartPre={ path=/usr/sbin/nginx ; argv[]=/usr/sbin/nginx -t ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; status=0/0 }
        ExecStart={ path=/usr/sbin/nginx ; argv[]=/usr/sbin/nginx ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; status=0/0 }
        ExecReload={ path=/bin/kill ; argv[]=/bin/kill -s HUP $MAINPID ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; status=0/0 }
        ...

    Examples:
        >>> systemctl_show_nginx["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_qpidd)
class SystemctlShowQpidd(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show qpidd``.

    Typical output of ``/bin/systemctl show qpidd`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:32 CST
        ExecMainStartTimestampMonotonic=104261679
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2747
        ExecMainCode=0
        ExecMainStatus=0
        ExecStart={ path=/usr/sbin/qpidd ; argv[]=/usr/sbin/qpidd --config /etc/qpid/qpi
        Slice=system.slice
        ControlGroup=/system.slice/qpidd.service
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_qpidd["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_qdrouterd)
class SystemctlShowQdrouterd(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show qdrouterd``.

    Typical output of ``/bin/systemctl show qdrouterd`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:32 CST
        ExecMainStartTimestampMonotonic=104261679
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2747
        ExecMainCode=0
        ExecMainStatus=0
        Slice=system.slice
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_qdrouterd["LimitNOFILE"]
        '4096'

    """
    pass


@parser(Specs.systemctl_smartpdc)
class SystemctlShowSmartpdc(SystemctlShow):
    """
    .. warning::
        This parser is deprecated, please use :py:class:`SystemctlShowServiceAll` instead.

    Class for ``systemctl show smart_proxy_dynflow_core``.

    Typical output of ``/bin/systemctl show smart_proxy_dynflow_core`` command is::

        Type=simple
        Restart=no
        NotifyAccess=none
        RestartUSec=100ms
        TimeoutStartUSec=1min 30s
        TimeoutStopUSec=1min 30s
        ExecMainStartTimestamp=Thu 2018-01-11 14:22:32 CST
        ExecMainStartTimestampMonotonic=104261679
        ExecMainExitTimestampMonotonic=0
        ExecMainPID=2747
        ExecMainCode=0
        ExecMainStatus=0
        Slice=system.slice
        LimitNOFILE=4096
        ...

    Examples:
        >>> systemctl_show_smartpdc["LimitNOFILE"]
        '4096'

    """
    pass



"""
Interrupts - file ``/proc/interrupts``
======================================

Provides parsing for contents of ``/proc/interrupts``.
The contents of a typical ``interrupts`` file looks like::

             CPU0       CPU1       CPU2       CPU3
    0:         37          0          0          0  IR-IO-APIC   2-edge      timer
    1:          3          2          1          0  IR-IO-APIC   1-edge      i8042
    8:          0          1          0          0  IR-IO-APIC   8-edge      rtc0
    9:      11107       2316       4040       1356  IR-IO-APIC   9-fasteoi   acpi
  NMI:        210         92        179         96   Non-maskable interrupts
  LOC:    7561411    2488524    6527767    2448192   Local timer interrupts
  ERR:          0
  MIS:          0

The information is parsed by the ``Interrupts`` class.  The information is stored
as a list of dictionaries in order corresponding to the output.
The counts in the CPU# columns are represented in a ``list``.
The following is a sample of the parsed information stored in an ``Interrupts``
class object::

    [
        { 'irq': '0',
          'num_cpus': 4,
          'counts': [37, 0, 0, 0],


        { 'irq': 'MIS',
          'num_cpus': 4,
          'counts': [0, ]}
    ]

Examples:
    >>> int_info = shared[Interrupts]
    >>> int_info.data[0]
    {'irq': '0', 'num_cpus': 4, 'counts': [37, 0, 0, 0],
     'type_device': 'IR-IO-APIC   2-edge      timer'}
    >>> int_info.num_cpus
    4
    >>> int_info.get('i8042')
    [{'irq': '1', 'num_cpus': 4, 'counts': [3, 2, 1, 0],
      'type_device': 'IR-IO-APIC   1-edge      i8042'}]
    >>> [i['irq'] for i in int_info if i['counts'][0] > 1000]
    ['9', 'LOC']
"""
from .. import Parser, parser
from ..parsers import ParseException
from insights.specs import Specs


@parser(Specs.interrupts)
class Interrupts(Parser):
    """Parse contents of ``/proc/interrupts``.

    Attributes:
        data (list of dict): List of dictionaries with each entry representing
            a row of the command output after the first line of headings.

    Raises:
        ParseException: Returned if first line is invalid, or no data is found to parse.
    """

    def get(self, filter):
        """list: Returns list of records containing ``filter`` in the type/device field."""
        return [i for i in self.data if 'type_device' in i and filter in i['type_device']]

    def __iter__(self):
        return iter(self.data)

    @property
    def num_cpus(self):
        """int: Returns total number of CPUs."""
        return int(self.data[0]['num_cpus'])

    def parse_content(self, content):
        self.data = []
        try:
            cpu_names = content[0].split()
        except:
            raise ParseException("Invalid first line of content for /proc/interrupts")
        if len(cpu_names) < 1 or not cpu_names[0].startswith("CPU"):
            raise ParseException("Unable to determine number of CPUs in /proc/interrupts")
        for line in content[1:]:
            parts = line.split(None, len(cpu_names) + 1)
            one_int = {'irq': parts[0].replace(":", "")}
            one_int['num_cpus'] = len(cpu_names)
            one_int['counts'] = []
            if len(parts) == len(cpu_names) + 2:
                one_int['type_device'] = parts[-1]
                for part, cpu in zip(parts[1:-1], cpu_names):
                    one_int['counts'].append(int(part))
            else:
                for part, cpu in zip(parts[1:], cpu_names):
                    one_int['counts'].append(int(part))
            self.data.append(one_int)
        if len(self.data) < 1:
            raise ParseException("No information in /proc/interrupts")



'''
NeutronPlugin - file ``/etc/neutron/plugin.ini``
================================================

The ``NeutronPlugin`` class parses the Neutron plugin configuration file.
See the ``IniConfigFile`` class for more usage information.
'''

from .. import parser, LegacyItemAccess, IniConfigFile
from insights.specs import Specs


@parser(Specs.neutron_plugin_ini)
class NeutronPlugin(LegacyItemAccess, IniConfigFile):
    """
    Parse the ``/etc/neutron/plugin.ini`` configuration file.

    Sample configuration file::

        [ml2]
        type_drivers = local,flat,vlan,gre,vxlan
        tenant_network_types = local,flat,vlan,gre,vxlan
        mechanism_drivers =openvswitch,linuxbridge
        extension_drivers =

        [ml2_type_flat]
        flat_networks =*
        # Example:flat_networks = physnet1,physnet2
        # Example:flat_networks = *

        [ml2_type_vlan]
        network_vlan_ranges =physnet1:1000:2999
        # Example: network_vlan_ranges = physnet1:1000:2999,physnet2

        [ml2_type_gre]
        tunnel_id_ranges =20:100

        [ml2_type_vxlan]
        vni_ranges =10:100
        vxlan_group =224.0.0.1

        [ml2_type_geneve]
        # (ListOpt) Comma-separated list of <vni_min>:<vni_max> tuples enumerating
        # ranges of Geneve VNI IDs that are available for tenant network allocation.
        #
        # vni_ranges =

        [securitygroup]
        enable_security_group = True

    Examples:

        >>> nconf = shared[NeutronPlugin]
        >>> 'ml2' in nconf
        True
        >>> nconf.has_option('ml2', 'type_drivers')
        True
        >>> nconf.get("ml2_type_flat", "flat_networks")
        "*"
        >>> nconf.items('ml2_type_vxlan')
        {'vni_ranges': '10:100',
         'vxlan_group': '224.0.0.1'}
    """
    pass



import doctest

from insights.combiners import sys_vmbus_devices
from insights.parsers.sys_vmbus import SysVmbusDeviceID, SysVmbusClassID
from insights.tests import context_wrap


DEVICE_ID_1 = """
{47505500-0001-0000-3130-444531444234}
""".strip()

CLASS_ID_1 = """
{44c4f61d-4444-4400-9d52-802e27ede19f}
""".strip()

DEVICE_ID_2 = """
{4487b255-b88c-403f-bb51-d1f69cf17f87}
""".strip()

CLASS_ID_2 = """
{3375baf4-9e15-4b30-b765-67acb10d607b}
""".strip()


def test_sys_vmbus_devices_combiner():
    result = sys_vmbus_devices.SysVmBusDeviceInfo(
        [
            SysVmbusDeviceID(context_wrap(DEVICE_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/device_id')),
            SysVmbusDeviceID(context_wrap(DEVICE_ID_2, path='/sys/bus/vmbus/devices/4487b255-b88c-403f-bb51-d1f69cf17f87/device_id')),
        ],
        [
            SysVmbusClassID(context_wrap(CLASS_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/class_id')),
            SysVmbusClassID(context_wrap(CLASS_ID_2, path='/sys/bus/vmbus/devices/4487b255-b88c-403f-bb51-d1f69cf17f87/class_id'))
        ]
    )

    assert result.devices[0].get('device_id') == '47505500-0001-0000-3130-444531444234'
    assert result.devices[0].get('class_id') == '44c4f61d-4444-4400-9d52-802e27ede19f'
    assert result.devices[0].get('description') == 'PCI Express pass-through'

    assert result.devices[-1].get('device_id') == '4487b255-b88c-403f-bb51-d1f69cf17f87'
    assert result.devices[-1].get('class_id') == '3375baf4-9e15-4b30-b765-67acb10d607b'
    assert result.devices[-1].get('description') == 'Unknown'


def test_documentation():
    result = sys_vmbus_devices.SysVmBusDeviceInfo(
        [
            SysVmbusDeviceID(context_wrap(DEVICE_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/device_id')),
            SysVmbusDeviceID(context_wrap(DEVICE_ID_2, path='/sys/bus/vmbus/devices/4487b255-b88c-403f-bb51-d1f69cf17f87/device_id')),
        ],
        [
            SysVmbusClassID(context_wrap(CLASS_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/class_id')),
            SysVmbusClassID(context_wrap(CLASS_ID_2, path='/sys/bus/vmbus/devices/4487b255-b88c-403f-bb51-d1f69cf17f87/class_id'))
        ]
    )

    failed_count, tests = doctest.testmod(
        sys_vmbus_devices,
        globs={
            'output': result
        }
    )
    assert failed_count == 0
